Struct UniqueRc

pub struct UniqueRc<T, A = Global>
where A: Allocator, T: ?Sized,
{ /* private fields */ }
🔬This is a nightly-only experimental API. (unique_rc_arc)
Available on crate feature wayland and Linux only.
A uniquely owned Rc.

This represents an Rc that is known to be uniquely owned – that is, have exactly one strong reference. Multiple weak pointers can be created, but attempts to upgrade those to strong references will fail unless the UniqueRc they point to has been converted into a regular Rc.

Because they are uniquely owned, the contents of a UniqueRc can be freely mutated. A common use case is to have an object be mutable during its initialization phase but then have it become immutable and converted to a normal Rc.

This can be used as a flexible way to create cyclic data structures, as in the example below.

#![feature(unique_rc_arc)]
use std::rc::{Rc, Weak, UniqueRc};

struct Gadget {
    #[allow(dead_code)]
    me: Weak<Gadget>,
}

fn create_gadget() -> Option<Rc<Gadget>> {
    let mut rc = UniqueRc::new(Gadget {
        me: Weak::new(),
    });
    rc.me = UniqueRc::downgrade(&rc);
    Some(UniqueRc::into_rc(rc))
}

create_gadget().unwrap();

An advantage of using UniqueRc over Rc::new_cyclic to build cyclic data structures is that Rc::new_cyclic’s data_fn parameter cannot be async or return a Result. As shown in the previous example, UniqueRc allows for more flexibility in the construction of cyclic data, including fallible or async constructors.

Implementations

Source

impl<T> UniqueRc<T>

pub fn new(value: T) -> UniqueRc<T>

🔬This is a nightly-only experimental API. (unique_rc_arc)

Creates a new UniqueRc.

Weak references to this UniqueRc can be created with UniqueRc::downgrade. Upgrading these weak references will fail before the UniqueRc has been converted into an Rc. After converting the UniqueRc into an Rc, any weak references created beforehand will point to the new Rc.

Source

impl<T, A> UniqueRc<T, A>
where A: Allocator,

pub fn new_in(value: T, alloc: A) -> UniqueRc<T, A>

🔬This is a nightly-only experimental API. (unique_rc_arc)

Creates a new UniqueRc in the provided allocator.

Weak references to this UniqueRc can be created with UniqueRc::downgrade. Upgrading these weak references will fail before the UniqueRc has been converted into an Rc. After converting the UniqueRc into an Rc, any weak references created beforehand will point to the new Rc.

Source

impl<T, A> UniqueRc<T, A>
where A: Allocator, T: ?Sized,

pub fn into_rc(this: UniqueRc<T, A>) -> Rc<T, A>

🔬This is a nightly-only experimental API. (unique_rc_arc)

Converts the UniqueRc into a regular Rc.

This consumes the UniqueRc and returns a regular Rc that contains the value that is passed to into_rc.

Any weak references created before this method is called can now be upgraded to strong references.

Source

impl<T, A> UniqueRc<T, A>
where A: Allocator + Clone, T: ?Sized,

pub fn downgrade(this: &UniqueRc<T, A>) -> Weak<T, A>

🔬This is a nightly-only experimental API. (unique_rc_arc)

Creates a new weak reference to the UniqueRc.

Attempting to upgrade this weak reference will fail before the UniqueRc has been converted to a Rc using UniqueRc::into_rc.